package de.ctdo.crashtest.game;

import de.ctdo.crashtest.domotics.*;
import de.ctdo.crashtest.gui.*;
import de.ctdo.crashtest.irc.*;

public class TheGame implements StateChangeListener, GuiEventListener, IRCEventListener {
    private IGuiControl guiControl;
    private IIrcClient ircClient;
    private IStatemachine machine;
    private IBuntiClient bunti;

    private int timertSeconds = 0;
    private int timertSecondsLast = 0;
    private int timeSpentTableGame = 0;
    private int timeSpentRokets = 0;



    public TheGame(IGuiControl guiControl) {
        this.guiControl = guiControl;

        this.ircClient = new IrcClient();
        this.machine =  new Statemachine();
        this.bunti = new BuntiClient("bunti.ctdo.de", 8080);

        initGame();
    }

    private void initGame() {
        guiControl.addListener(this);
        ircClient.addListener(this);
        machine.addStateChangedListener(this);

        machine.reset();
    }


    /**
     * Event listener for state change events from statemachine
     * @param newState  the new game state from statemachine
     */
    @Override
    public void stateChanged(Statemachine.state newState) {

        ircClient.say("New State: " +  newState);

        switch (newState) {
            case IDLE:
                bunti.setPar56(0,0,0);
                bunti.setLampel(false,false,false);
                guiControl.showCountDown(false);
                break;
            case ENTERED_ROOM:
                bunti.setLampel(false,false,false);
                bunti.setPar56(20,0,100);
                break;
            case TABLE_GAME_ONE:
                bunti.setLampel(true,false,false);
                guiControl.showCountDown(true);
                bunti.setPar56(255,0,100);
                break;
            case TABLE_GAME_TWO:
                bunti.setLampel(false,true,false);
                bunti.setPar56(255,0,100);
                break;
            case TABLE_GAME_THREE:
                bunti.setLampel(false,true,false);
                bunti.setPar56(255,35,0);
                break;
            case TABLE_GAME_FOUR:
                bunti.setLampel(false,true,false);
                bunti.setPar56(255,55,0);
                break;
            case TABLE_GAME_FIVE:
                bunti.setLampel(false,true,false);
                bunti.setPar56(255,75,0);
                break;
            case TABLE_GAME_SIX:
                bunti.setLampel(false,true,false);
                bunti.setPar56(255,100,0);
                break;
            case TABLE_GAME_SEVEN:
                bunti.setLampel(false,false,true);
                bunti.setPar56(255,100,0);
                break;
            case TABLE_FINISH:  // und roket muss starten
                bunti.setLampel(false,false,true);
                bunti.setPar56(0, 255, 0);
                break;
            case ROKET_DONE:
                bunti.setLampel(true,true,true);
                bunti.setPar56(255, 196, 0);
                break;
        }

    }


    /**
     * Event listener for keyPress events from the GUI
     * @param key  the pressed key
     */
    @Override
    public void keyPress(char key) {
        machine.handleInput(key);
        guiControl.setExtra("btn: " + key);

    }

    /**
     * Event lister for window closing event from the GUI
     */
    @Override
    public void windowClosing() {

    }

    /**
     * Event listener for IRC Messages
     * @param message the message from the IRC user
     */
    @Override
    public void handleMessage(final String message) {
        if(message.startsWith("help")) {
            handleHelpCommand(message);
        } else if(message.equals("reset")) {
            machine.reset();
            ircClient.say("reset done");
        } else if(message.startsWith("state")) {
            handleStateCommand(message);
        } else if(message.startsWith("timer")) {
            handleTimerCommand(message);
        } else if(message.startsWith("score")) {
            handleScoreCommand(message);
        } else if(message.startsWith("wall")) {
            guiControl.setWall(message.substring("wall".length()).trim());
        } else if(message.startsWith("extra")) {
            guiControl.setExtra(message.substring("extra".length()).trim());
        } else {
            ircClient.say("y u no use valid command?");
        }
    }

    private void handleTimerCommand(final String message) {
        String params = message.substring("timer".length()).trim().toLowerCase();

        if(params.startsWith("start")) {
            String timeStr = params.substring("start".length()).trim();

            if(timeStr.length() > 0) {
                int time = 0;
                
                try {
                    time = Integer.parseInt(timeStr);

                    guiControl.showCountDown(true);
                    ircClient.say("got it, starting with " + time + " seconds");
                }
                catch (NumberFormatException e) {
                    ircClient.say("your number looks a bit odd...");
                }
            } else {
                ircClient.say("invalid parameter");
            }            
        } else if(params.startsWith("stop")) {

            guiControl.showCountDown(false);
            ircClient.say("timer stopped");
        } else if(params.startsWith("pause")) {
            
        } else if(params.startsWith("resume")) {
            
        }
               

    }

    private void handleStateCommand(final String message) {
        if(message.equals("state")) {
            ircClient.say(machine.getCurrentState().name());
        } else {

            String params = message.substring("state".length()).trim().toLowerCase();

            Boolean ok = false;
            for(Statemachine.state st: Statemachine.state.values()) {
                String stateName = st.name().toLowerCase();
                if(stateName.equals(params)) {
                    ok = true;
                    machine.setNewState(st);
                    ircClient.say("consider it done, sir!");
                    break;
                }
            }

            if(!ok) ircClient.say("ehm, impossibruu!");
            
        }
    }

    private void handleScoreCommand(final String message) {
        ircClient.say("stateChangeCounter: " + machine.getStateChangeCounter());
        ircClient.say("timerlast: " + timertSecondsLast / 10);
    }

    
    private void handleHelpCommand(final String message) {
        if(message.equals("help")) {
            ircClient.say("commands: help, reset, state, timer, wall, extra, score");
        } else if(message.contains("reset")) {
            ircClient.say("resets the game to IDLE");
        } else if(message.contains("state")) {
            ircClient.say("get or set game state");
            ircClient.say("> state <NEWSTATE>");
            ircClient.say("valid states: ");

            int counter = 0;
            StringBuilder sb = new StringBuilder();
            for(Statemachine.state st: Statemachine.state.values()) {
                sb.append(st.name());
                sb.append("    ");
                if(++counter == 4) {
                    counter = 0;
                    ircClient.say(sb.toString());
                    sb.setLength(0);
                }
            }

            if(sb.length() > 0) {
                ircClient.say(sb.toString());
            }
        } else if(message.contains("timer")) {
            ircClient.say("control timer. commands are: start, stop, pause, resume.");
            ircClient.say("for start give lenght in seconds as parameter");
            ircClient.say("e.g.: timer start 120");
        } else if(message.contains("wall")) {
            ircClient.say("set text message on the screen");
        } else if(message.contains("extra")) {
            ircClient.say("set small extra message on the screen");
        } else if(message.contains("score")) {
            ircClient.say("i will tell you the current game score");
        } else {
            ircClient.say("dafuq?");
        }

    }

}
